@@ -277,8 +277,9 @@ static char *romajimap1[]=
 	"wa","wo",
 	"ya","yu","yo",
 	"a","i","u","e","o",
-	"-", "'",
+	"-", "'","(",")",
 	""
+	/* See also hardcodes for special keys in Handle_romajikana */
 };
 /* must match order of romajimap1[]!!*/
 static XChar2b romajimap2[150][3]=
@@ -418,6 +419,9 @@ static XChar2b romajimap2[150][3]=
 
 	{{0x21, 0x3c}, {0, 0}},/*- for elongation*/	
 	{{0x24, 0x43}, {0, 0}},/*"'" for small-tsu*/
+		/* These two must match ReadProdunciation() */
+	{{0x21, 0x4a}, {0, 0}},/*"(" for open-paren. */
+	{{0x21, 0x4b}, {0, 0}},/*")" for close-paren. */
 
 	{0}
 	
@@ -537,7 +541,10 @@ void Handle_romajikana(Widget w, XtPointer closure, XEvent *e, Boolean *cont)
 	}
 	key_event = (XKeyEvent *) e;
 	
-	inbetweenK = XKeycodeToKeysym(XtDisplay(w), key_event->keycode,0);
+	/*inbetweenK = XKeycodeToKeysym(XtDisplay(w), key_event->keycode,0);*/
+	/* need XtGetActionkeysym to detect Shift-9=='(' */
+	inbetweenK = XtGetActionKeysym(e, NULL);
+
 	if(inbetweenK == (KeySym)NULL)
 	{
 		puts("NULL keysym on kana input???");
@@ -547,6 +554,8 @@ void Handle_romajikana(Widget w, XtPointer closure, XEvent *e, Boolean *cont)
 	/* we switch based on what character has just been
 	 * pressed on an ASCII-based keyboard.
 	 * <return> gets translated to "go search now"
+	 * Making byte1== 0x23, is a special signal to romajitokana,
+	 * that says "Hey, this is romaji"
 	 */
 	switch(inbetweenK)
 	{
@@ -586,6 +595,23 @@ void Handle_romajikana(Widget w, XtPointer closure, XEvent *e, Boolean *cont)
 		addchar.byte2 = '\'';	/* 0x3e*/
 		process_kinput(addchar);
 		return;
+
+	   case XK_parenleft:
+		/* hack to get "small tsu" to work.
+		 * Note that this "char" is invisible.
+		 */
+		addchar.byte1 = 0x23;
+		addchar.byte2 = '(';	/* 0x3e*/
+		process_kinput(addchar);
+		return;
+	   case XK_parenright:
+		/* hack to get "small tsu" to work.
+		 * Note that this "char" is invisible.
+		 */
+		addchar.byte1 = 0x23;
+		addchar.byte2 = ')';	/* 0x3e*/
+		process_kinput(addchar);
+		return;
 		
 	}
 	charpressed = XKeysymToString(inbetweenK);
@@ -600,10 +626,12 @@ void Handle_romajikana(Widget w, XtPointer closure, XEvent *e, Boolean *cont)
 	if((*charpressed <0x61) || (*charpressed >0x7a))
 	{
 		/* outside range of ascii chars we like */
+		if((*charpressed!='(') && *charpressed != ')'){
 #ifdef DEBUG
-		puts("ignoring.. not in normal ascii range");
+			puts("ignoring.. not in normal ascii range");
 #endif
-		return;
+			return;
+		}
 	}
 	
 	addchar.byte1 = 0x23;
