@@ -1,21 +1,19 @@
 /* GLOBAL externs file for kdrill!!
  * Hopefully, the only stuff that is in here, is stuff that should REALLY
  * be globally visible, not just any stuff that happens to be exported
- * by a particular file/module
+ * by a particular file/module.
  */
 
 extern Atom delete_message,wm_message;
 
 
 
-
-
 extern TRANSLATION translations[MAXTRANSLATIONSALLOWED];
 
 extern int lowestkanji,highestkanji,numberofkanji;/* keep track of how many we have */
 extern int lowfrequency,highfrequency;		/* for frequency restrictions */
 
-extern char *usefile;
+extern char usefilename[]; /*actual name of usefile*/
 
 extern void usage();
 extern void quit(Widget w, XtPointer, XtPointer);
@@ -23,17 +21,22 @@ extern void Beep();	/* nice generic routine that respects whether
 			 * player wants quiet or not
 			 */
 
-extern void MakeFrequency();
-
 extern void SearchCallback(Widget,XtPointer,XtPointer);
 
 
-/* widget operators.. */
+/* general-purpose widget operators.. */
 extern int GetWidgetNumberval(Widget);
+extern void ReverseButton(Widget);
 extern void UnreverseButton(Widget);
+extern void SetWhiteOnBlack(Widget);
+extern void SetBlackOnWhite(Widget);
 extern void SetWidgetNumberval(Widget,int);
 extern void SetWidgetHexval(Widget,int);
-extern void ReverseButton(Widget);
+
+/* other "usefull" general functions */
+extern int xtoi(char *);
+extern int getline(FILE *, unsigned char *);
+extern void setstatus(char *);
 
 extern int FindIndex(char *);
 
@@ -41,17 +44,16 @@ extern int FindIndex(char *);
 extern XFontStruct *largekfont;
 extern XFontStruct *smallkfont;
 extern XFontStruct *englishfont;
+extern XFontStruct *defaultfont;
 
-/* other "usefull" generic functions */
-extern int xtoi(char *), getline(FILE *, unsigned char *);
-extern void setstatus(char *);
 
 
 /* callbacks have been moved to their specific .h files */
 
-
 extern XtAppContext Context;
 
+extern char *homedir;
+
 extern unsigned long black,white;
 extern Display	*display;
 extern Window	mainwindow,rootwindow;
@@ -77,6 +79,9 @@ extern void DoFind(Widget,XEvent *,String *,Cardinal *);
 extern void GetXtString(char *, char *, char *);
 extern int GetXtNumber(char *,char *);
 extern Boolean GetXtBoolean(char *, char *);
+extern void SetXtString(char *, char *);
+extern void SetXtNumber(char *,int);
+extern void SetXtBoolean(char *, Boolean);
 
 /* These flags indicate current settings of the game */
 extern Boolean useUsefile; /*	we Always read in a usefile, but
