@@ -23,6 +23,7 @@
 #include "init.h"
 #include "game.h"
 #include "convert.h"
+#include "badguess.h"
 
 TRANSLATION lastpicked=NULL;/* we can't "really" auto-initialize this...
 		  * set to zero so we can set it
@@ -49,17 +50,21 @@ void SetupGuess();
 /*
 *UseThisKanji()
 *	Returns Boolean value on whether this is a "valid" kanji for us,
-*	based on usefile markings,
-*	current grade set,
-*	and whether it has english/kana translation
+*	based on usefile markings, current grades allowed,
+*	and whether it has english/kana translation available
 */
-Boolean UseThisKanji(TRANSLATION kanji)
+Boolean UseThisKanji(int kanjinum)
 {
+	TRANSLATION kanji = translations[kanjinum];
 
-	/*usefile cancels it? */
-
-/* FIX THIS FOR v5 */
-/*	if(useUsefile && (useKanji[kanjinum] == 0)) return False;*/
+	
+	if(useUsefile)
+	{
+		if(InUsefile(kanjinum) == False)
+		{
+			return False;
+		}
+	}
 	
 	/* nonexistant kanji? */
 	if(kanji == NULL) return False;
@@ -122,25 +127,47 @@ Boolean UseThisKanji(TRANSLATION kanji)
  *	 (the secnd being because kanjidic does not always have
  *	 english and/or kana listings
  *
- *   This counts the number of usable kanji.
- *   It ALSO has to update the "numberincorrect" counter;
+ *   This counts the number of usable (enabled) kanji.
+ *
+ *  DO NOT update "numberincorrect" here. It is independant of
+ *  whether or not a kanji is active this second or not.
+
  */
 void CountKanji(){ 
-	TRANSLATION count;
+	TRANSLATION ktrans;
+	int knum;
 	numberofkanji=0;
-	numberincorrect=0;
 
-	count = translations[lowestkanji];
-	while(count != NULL){
-		if(UseThisKanji(count)){
+	for(knum=lowestkanji; knum<=highestkanji;knum++)
+	{
+		ktrans = translations[knum];
+		if(ktrans == NULL)
+			continue;
+		if(UseThisKanji(knum)){
 			numberofkanji++;
-			if(count->incorrect >0)
-				numberincorrect++;
 		}
-		count=count->nextk;
 	}
 }
 
+/* kinda like CountKanji, but this ONLY updates 'numberincorrect' 
+ * This is a brute force util, to recount when/if we have a leak in
+ * our 'numberincorrect' calculations
+ */
+void TallyWrong()
+{
+	int wcount=0;
+	int kcount=0;
+	for(kcount=lowestkanji; kcount<=highestkanji; kcount++)
+	{
+		if(translations[kcount]==NULL)
+			continue;
+		if(translations[kcount]->incorrect>0)
+			wcount++;
+	}
+	numberincorrect=wcount;
+	SetWidgetNumberval(kanjiMissed, numberincorrect);
+}
+
 /* pickkanji:
  *   picks an acceptably random kanji index.
  *	NOTE THAT THIS DEPENDS ON "numberofkanji" being correct!
@@ -153,7 +180,7 @@ TRANSLATION pickkanji(){
 	rand_kanji = lrand48()%numberofkanji;
 
 	for(count=lowestkanji;count<=highestkanji;count++) {
-		if(UseThisKanji(translations[count])) {
+		if(UseThisKanji(count)) {
 		      rand_kanji--;
 		      if(rand_kanji <0)
 			  return translations[count];
@@ -168,10 +195,16 @@ TRANSLATION pickkanji(){
 }
 
 /* pickincorrect:
- *	pick a kanji we have missed previously.
- *	start at first incorect kanji after the last one picked.
+ *	This would be more accurately named "Pick one you missed previously"
+ * 
+ *	Start at first incorect kanji after the last one picked,and look
+ *	for another missed one.
  *	This hopefully ensures that we don't replay the same incorrect
  *	one too often.
+ *
+ *  We are trying to phase this routine out, in favour of
+ *    GetRepeat()
+ *
  */
 TRANSLATION pickincorrect()
 {
@@ -195,16 +228,18 @@ TRANSLATION pickincorrect()
 				count = lowestkanji;
 			currentpick=translations[count];
 
-		} while(!UseThisKanji(currentpick));
+		} while(!UseThisKanji(count));
 		if(translations[count] == lastpicked)
 		{
-			setstatus("Internal error picking repeat");
+			if(numberincorrect>1)
+			{ /* why didnt we hit a second incorrect one? */
+				setstatus("Internal error picking repeat");
+			}
 			return lastpicked;
 		}
 
 	} while (currentpick->incorrect == 0);
 
-	setstatus("You missed this one before");
 	return currentpick;
 }
 
@@ -229,8 +264,9 @@ TRANSLATION picktruevalue(){
 		/* First check if they have missed any */
 		if(numberincorrect>0){
 			if(lrand48()%REPEATFRACTION == 0){
-				/* go back */
-				ret_val = pickincorrect();
+				/* repeat what user missed  */
+				/*ret_val = pickincorrect();*/
+				ret_val=GetRepeat();
 			}
 			else do {
 				ret_val = pickkanji();
@@ -269,19 +305,35 @@ TRANSLATION picktruevalue(){
 				currentpick = lowestkanji;
 		
 
-		} while(!UseThisKanji(translations[currentpick]));
+		} while(!UseThisKanji(currentpick));
 
 		ret_val = translations[currentpick];
 	}
 	return ret_val;
 } 
 
+/* If player has mis-recognized a kanji, call this routine to 
+ * make a record of that.
+ *
+ * Called by guessvalue()
+ */
+void markasmissed(TRANSLATION kanjiP){
+	if(kanjiP->incorrect ==0)
+		numberincorrect+=1;
+	kanjiP->incorrect += REPEATTIMES;
+	if(kanjiP->incorrect >MAXREPEAT)
+		kanjiP->incorrect = MAXREPEAT;
+	AdjustBadCache(kanjiP);
+}
 
 /*guessvalue(int)
- *	passed number by EITHER kanji button or english button,
+ *	This is what handles user guesses.
+ *	Passed in number by EITHER kanji button or english button,
  *	we don't care.
  *	Set status bar to correct/incorrect. Beeps if incorrect
- *	Also sets "incorrect" counter in the translations[].
+ *	Also sets "incorrect" counter in the translations[],
+ *	for the translation wrongly guessed.
+ *	ALSO calls routine to set total wrong.
  *	
 */
 /* note that "-1" on guess means "you lose: ran out of time" */
@@ -298,8 +350,14 @@ void guessvalue(int guess){
 				break;
 			case 1:
 				numberincorrect -= 1;
+				/* and fall through */
 			default:
 				kanjiP->incorrect -= 1;
+				AdjustBadCache(kanjiP);
+				SetWidgetNumberval(kanjiMissed,
+							   numberincorrect);
+				
+				/*	TallyWrong();*/
 		}
 		SetupGuess();
 		return;
@@ -313,20 +371,14 @@ void guessvalue(int guess){
 	Beep();
 	setstatus("Incorrect.");
 	if(guess != -1) {
-		if(kanjiP->incorrect ==0)
-			numberincorrect+=1;
-		kanjiP->incorrect += REPEATTIMES;
-		if(kanjiP->incorrect >MAXREPEAT)
-			kanjiP->incorrect = MAXREPEAT;
+		markasmissed(kanjiP);
 	}
 
 	kanjiP = values[truevalue];
-	if(kanjiP->incorrect ==0)
-		numberincorrect+=1;
-	kanjiP->incorrect += REPEATTIMES;
-	if(kanjiP->incorrect >MAXREPEAT)
-		kanjiP->incorrect = MAXREPEAT;
-	
+	markasmissed(kanjiP);
+
+	SetWidgetNumberval(kanjiMissed, numberincorrect);
+
 }
 
 
@@ -335,13 +387,13 @@ void guessvalue(int guess){
  *	Sets widget label to be kanji string.
  *	Will set font.
  */
-void printkanji(Widget w,TRANSLATION knum){
+void printkanji(Widget w,TRANSLATION trans){
 
 	XtVaSetValues(w,	XtNencoding,XawTextEncodingChar2b,
 				XtNfont,largekfont,
 				NULL);
 
-	XtVaSetValues(w,	XtNlabel,knum->kanji,
+	XtVaSetValues(w,	XtNlabel,trans->kanji,
 				NULL);
 		      
 
@@ -352,13 +404,13 @@ void printkanji(Widget w,TRANSLATION knum){
  *	Sets widget label to appropriate english.
  *	Will change fonts to englishfont.
  */
-void printenglish(Widget widget,TRANSLATION Tnum)
+void printenglish(Widget widget,TRANSLATION trans)
 {
 
 	XtVaSetValues(widget,
 		     XtNencoding,XawTextEncoding8bit,
 		     XtNfont,englishfont,
-		     XtNlabel,Tnum->english,
+		     XtNlabel,trans->english,
 		     NULL);
 
 }
@@ -369,13 +421,19 @@ void printenglish(Widget widget,TRANSLATION Tnum)
  *	Will change font to smallkfont
  */
 
-void printkana(Widget widget,TRANSLATION Tnum){
+void printkana(Widget widget,TRANSLATION trans){
 	if(romajiswitch ==1)
 	{
-		char stringbuff[100];
+		char stringbuff[MAXROMAJI+1];
 		/*translate all kana into romaji */
 		/* translate to buffer, then set widget string*/
-		kanatoromaji(Tnum->pronunciation, stringbuff);
+		kanatoromaji(trans->pronunciation, stringbuff);
+
+		/* stupid X bug workaround... */
+		XtVaSetValues(widget,
+		      XtNlabel,"",
+		      NULL);
+		
 		XtVaSetValues(widget,
 		     XtNencoding,XawTextEncoding8bit,
 		     XtNfont,englishfont,
@@ -392,7 +450,7 @@ void printkana(Widget widget,TRANSLATION Tnum){
 		     XtNfont,smallkfont,
 		     NULL);
 		XtVaSetValues(widget,
-		      XtNlabel,Tnum->pronunciation,
+		      XtNlabel,trans->pronunciation,
 		      NULL);
 	}
 }
@@ -431,7 +489,7 @@ void printallchoices(){
 	int i;
 
 	for(i=0;i<NUMBEROFCHOICES;i++){
-		UnreverseButton(choicesWidgets[i]);
+		SetBlackOnWhite(choicesWidgets[i]);
 
 		switch(choicesmode){
 			case GUESS_ENGLISH:
@@ -514,6 +572,9 @@ void SetupGuess()
 	printgrades();	/* stupid buttons needs updating manually,
 			 * EVERY durn time. arrg */
 
+	if(values[truevalue]->incorrect>0)
+		setstatus("You missed this one before");
+
 }
 
 
