@@ -13,6 +13,8 @@ extern  TRANSLATION values[NUMBEROFCHOICES];
 extern void CountKanji();
 extern void SetupGuess();
 
+extern TRANSLATION pickkanji(); /* only for use from badguess.c */
+
 extern void BackCallback(Widget,XtPointer,XtPointer);
 extern void ChangeMode(Widget,XtPointer,XtPointer);
 extern void Guessvalue(Widget,XEvent *,String *,Cardinal *);
@@ -26,7 +28,7 @@ extern void printallchoices(), printquestion();
 
 extern void guessvalue(int);
 
-extern Boolean UseThisKanji(TRANSLATION kanjinum);
+extern Boolean UseThisKanji(int kanjinum);
 
 /* This function is declared in externs.h, but should be here, sort of
 *    extern void setstatus(char *);
