@@ -24,21 +24,18 @@ int gradelevelflags;
 
 
 
-/* normally, I would make this inline, but
- * " grades.c:20: warning: ANSI does not permit the keyword `inline' "
- * Hurmph....
- *   So, This routine reverses black/white values of a widget
+/*
+ * Sets widget to have white text on a black background
  */
-void ReverseButton(Widget w){
+void SetWhiteOnBlack(Widget w){
 	XtVaSetValues(w,XtNbackground,black,
 		       XtNforeground,white,
 		       NULL);
 }
-/* DO NOT make this inline ...
- * used elsewhere.
- * Undoes what ReverseButton() does
+/*
+ * Sets widget to have black text on a white background
  */
-void UnreverseButton(Widget w){
+void SetBlackOnWhite(Widget w){
 	XtVaSetValues(w,XtNbackground,white,
 		       XtNforeground,black,
 		       NULL);
@@ -53,9 +50,9 @@ void printgrades(){
 	int buttonloop;
 	for(buttonloop=1;buttonloop<8;buttonloop++){
 		if(gradelevelflags & (1<<buttonloop)){
-			ReverseButton(gradeButtons[buttonloop]);
+			SetWhiteOnBlack(gradeButtons[buttonloop]);
 		} else {
-			UnreverseButton(gradeButtons[buttonloop]);
+			SetBlackOnWhite(gradeButtons[buttonloop]);
 		}
 	}
 }
@@ -88,6 +85,26 @@ void parsegrades(char *gradestring){
 		gradelevelflags = ALLGRADES;
 }
 
+
+/* routine for GradeCallback.
+ * This gets called whenever we CHANGE our grade selections.
+ * We then update our preference database
+ */
+void calcgradestring() {
+	char gradestring[10];
+	gradestring[0]='\0';
+
+	/* Yes, this is messed up. oh well, it works */
+	if(gradelevelflags&0x02) strcat(gradestring,"1");
+	if(gradelevelflags&0x04) strcat(gradestring,"2");
+	if(gradelevelflags&0x08) strcat(gradestring,"3");
+	if(gradelevelflags&0x10) strcat(gradestring,"4");
+	if(gradelevelflags&0x20) strcat(gradestring,"5");
+	if(gradelevelflags&0x40) strcat(gradestring,"6");
+	if(gradelevelflags&0x80) strcat(gradestring,"+");
+
+	SetXtString("gradelevel",gradestring);
+}
 /* GradeCalback:
  *	calback for different grade buttons
  */
@@ -120,7 +137,7 @@ void GradeCallback(Widget w,XtPointer data,XtPointer calldata){
 						return;
 						
 					}
-					UnreverseButton(w);
+					SetBlackOnWhite(w);
 					if((int) data <6)
 						sprintf(statusprefix,"Grade %d unset: ",(int) data);
 					else
@@ -129,7 +146,7 @@ void GradeCallback(Widget w,XtPointer data,XtPointer calldata){
 				/* CountKanji already called above */
                    	} else {
 			/* nope.. turn on grade level */
-				ReverseButton(w);
+				SetWhiteOnBlack(w);
 				gradelevelflags |= grademask;
 					if((int) data <6)
 						sprintf(statusprefix,"Grade %d set: ",(int) data);
@@ -147,6 +164,7 @@ void GradeCallback(Widget w,XtPointer data,XtPointer calldata){
 			break;
 	}
 	sprintf(statusbuff,"%s%d kanji active",statusprefix,numberofkanji);
+	calcgradestring();/* update preferences */
 	setstatus(statusbuff);
 }
 
