@@ -23,6 +23,7 @@
 
 static char *version = VERSION;
 
+char *homedir=NULL;
 Display *display;
 int screen;
 Window mainwindow,rootwindow;
@@ -64,14 +65,15 @@ void clean_up_structs(){
 /* quitbutton, of course */
 void quit(Widget w,XtPointer data,XtPointer calldata)
 {
-#ifdef DEBUG
-	puts("quitting?");
-#endif
+
+	/*puts("quitting?");*/
+
+	/* rely on this to do the right thing in all cases */
+	MakeLog(NULL, NULL, NULL);
+	SaveUsefile();
+	SaveConfig();
 	XtCloseDisplay(display);
-#ifdef OLDLOG
-	if(logfile != NULL)
-		fclose(logfile);
-#endif
+
 	/* I supose we'll be nice an' clean up memory somewhat.. */
 	clean_up_structs();
 	exit(0);
@@ -114,87 +116,6 @@ void usage(){
 }
 
 
-/* GetXtNumber(), GetXtBoolean(), GetXtString():
- *	Not the "best way" to get resources...
- *	supposed to read them all in at one go.
- *	But I had reasons for reading things in at different times,
- *	and I have stuck to that model
- */
-
-/* GetXtString:
- *	Given the resource name and class, will
- *	 copy the resource string value to
- *	 destinationstring address.
- *	Destination must be MALLOCED ARRAY!!!
- *
- *	Default value of string is "NOT SET".
- *	This is used by initlog() [which is not used any more]
- *	This should actually be encountered. I provide default values for
- *	all resource strings used.
- */
-void GetXtString(char *name,char *rec_class, char *destinationstring)
-{
-	char resourcename[100],resourceclass[100];
-	char buffer[100];
-	char *returnstring = buffer;
-	
-
-	XtResource resourceList ={
-		NULL,NULL,
-		XtRString,sizeof(char *),0,
-		XtRString,"NOT SET"
-	};
-	resourceList.resource_name = resourcename;
-	resourceList.resource_class = resourceclass;
-
-	strcpy(resourcename,name);
-	strcpy(resourceclass,rec_class);
-
-	XtGetApplicationResources(toplevel,&returnstring,
-				  &resourceList,1,
-				  NULL,0);
-	strcpy(destinationstring,returnstring);
-}
-
-/* GetXtNumber:
- *	Given resource name and and class, will attempt to look up
- *	resource value in database.
- *	Will return default of 0!!!
- */
-int GetXtNumber(char *name,char *rec_class)
-{
-	char resourcevalue[100];
-
-	GetXtString(name,rec_class,resourcevalue);
-	return atoi(resourcevalue);
-}
-/* GetXtBoolean()
- *	see GetXtNumber.
- *
- *	default of False
- */
-Boolean GetXtBoolean(char *name,char *rec_class)
-{
-	Boolean resourcevalue;
-	char resourcename[100],resourceclass[100];
-
-	XtResource resourceList ={
-		NULL,NULL,
-		XtRBoolean,sizeof(Boolean),0,
-		XtRBoolean,False,
-	};
-	resourceList.resource_name = resourcename;
-	resourceList.resource_class = resourceclass;
-
-	strcpy(resourcename,name);
-	strcpy(resourceclass,rec_class);
-
-	XtGetApplicationResources(toplevel,&resourcevalue,
-				  &resourceList,1,
-				  NULL,0);
-	return resourcevalue;
-}
-
 /* debugging routine.
  * Prints out ALL english phrases loaded into memory.
  * Note: This can be many thousands of lines!!!
@@ -224,14 +145,16 @@ void dumpenglish()
  */
 int main(int argc,char *argv[])
 {
-	puts("kdrill: Starting up... please wait a while.");
+	printf("kdrill %s: by Philip Brown -- phil@bolthole.com\n",
+	       VERSION);
+	puts("Starting up... please wait a while.");
 	fflush(stdout);
 
 	/* Widgets get made down in initstuffs()... */
 	/* also, resources get called */
 	initstuffs(&argc,argv);
 
- /*	ReverseButton(form);*/
+ /*	SetBlackOnWhite(form);*/
 
 
 	/* check for abridgements of the dictionary... */
@@ -245,10 +168,6 @@ int main(int argc,char *argv[])
 	
 	initlog(); /* old way or new way: same name */
 
-	if( GetXtBoolean("showkana","Showkana"))
-		choicesmode = GUESS_KANA;
-	if( GetXtBoolean("romajiswitch","Romajiswitch"))
-		romajiswitch = 1;
 	CountKanji();
 	if(numberofkanji<HAVE_AT_LEAST){
 		fprintf(stderr,"There are two few kanji readable in the current configuration\n");
@@ -259,6 +178,13 @@ int main(int argc,char *argv[])
 
 	
 	SetupGuess();
+	TallyWrong();
+
 	XtAppMainLoop(Context);
 	return 0;
 }
+
+
+
+
+
