@@ -63,18 +63,23 @@ void ChangeMode(Widget w,XtPointer data,XtPointer calldata)
 #endif
 	switch(buttonpressed){
 		case GUESS_ENGLISH:
+			SetXtString("guessmode", "english");
+			break;
 		case GUESS_KANJI:
+			SetXtString("guessmode", "kanji");
 		case GUESS_KANA:
-			choicesmode = buttonpressed;
-			if(switchKanaEnglish){ /* allow switching in place */
-				printallchoices();
-			} else {
-				CountKanji();
-				SetupGuess();
-			}
+			SetXtString("guessmode", "kana");
 			break;
 		default:
 			fprintf(stderr,"Error: cannot switch to non-existant mode\n");
+			return;
+	}
+	choicesmode = buttonpressed;
+	if(switchKanaEnglish){ /* allow switching in place */
+		printallchoices();
+	} else {
+		CountKanji();
+		SetupGuess();
 	}
 }
 
