@@ -24,6 +24,7 @@
 Widget orderbutton,usefilebutton;
 Widget kanjimode,englishmode,kanamode;
 
+int buttonswidth=BUTTONSWIDTH;
 
 Widget options_popup;
 Widget optionsform;
@@ -51,36 +52,53 @@ void ordercallback(Widget w,XtPointer data,XtPointer calldata)
 	   switch(showinorder){
 		case 0:
 			setstatus("Will now select kanji randomly");
+			SetXtBoolean("showinorder", False);
 			break;
 		case 1:
 			setstatus("Will now go through kanji in order of #");
+			SetXtBoolean("showinorder", True);
 			break;
 
 	   }
 	   return;
 }
-		   
-void usefilecallback(Widget w,XtPointer data,XtPointer calldata)
+
+/* update the LABEL of the "using usefile" button in options window,
+ * according to global useUsefile
+ */
+void UpdateUsefileL()
+{
+	XtVaSetValues(usefilebutton,XtNlabel,usefilelabel[(int) useUsefile],NULL);
+}
+
+void UsefileCallback(Widget w,XtPointer data,XtPointer calldata)
 {
 	char statbuff[300];
 	   switch(useUsefile){
-		case 1:
-			useUsefile = 0;
-			if(usefile != NULL)
-				setstatus("Will now ignore usefile");
+		case True:
+			useUsefile = False;
+			SetXtBoolean("nousefile", True);
+
+			setstatus("No longer restricting by usefile");
 			break;
-		case 0:
-			if(usefile == NULL){
-				setstatus("No usefile to use");
-				break;
+		case False:
+			if(NumInUsefile() <HAVE_AT_LEAST)
+			{
+				sprintf(statbuff,"only %d entries: cannot use",
+						NumInUsefile());
+				setstatus(statbuff);
+				return;
 			}
-			useUsefile = 1;
-			sprintf(statbuff,"now using usefile %s",
-				usefile);
+			
+			SetXtBoolean("nousefile", False);
+			useUsefile = True;
+			sprintf(statbuff,"Now using usefile %s",
+				usefilename);
 			setstatus(statbuff);
 			break;
 	   }
-	XtVaSetValues(w,XtNlabel,usefilelabel[(int) useUsefile],NULL);
+
+	UpdateUsefileL();
 	CountKanji();
 	return;
 }
@@ -91,19 +109,28 @@ void usefilecallback(Widget w,XtPointer data,XtPointer calldata)
  */
 void romajicallback(Widget w,XtPointer data,XtPointer calldata)
 {
+	romajiswitch=1-romajiswitch;
+	
 	switch(romajiswitch){
 		case 1:
-			romajiswitch = 0;
-			setstatus("Will now print kana");
+			setstatus("Will now print romaji");
+			SetXtBoolean("romajiswitch", True);
 			break;
 		case 0:
-			romajiswitch = 1;
-			setstatus("Will now print romaji");
+			setstatus("Will now print kana");
+			SetXtBoolean("romajiswitch", False);
 			break;
 	}
 	XtVaSetValues(w,XtNlabel,romajilabel[(int)romajiswitch],NULL);
+
+	/* Now go and reset ALL labels. Sigh. */
+	printquestion();
 	printallchoices();
+	setmultilabels();
+	SearchRefresh(); /* search window */
+	/* kana popup window! */
 	ksearch_toggles(w, (XtPointer)NULL, (XtPointer)1);
+
 	return;
 }
 		   
@@ -113,98 +140,33 @@ void romajicallback(Widget w,XtPointer data,XtPointer calldata)
  */
 void OptionsCallback(Widget w,XtPointer client_data, XtPointer call_data)
 {
-	static up = -1;
+	static options_up = -1;
 	static Position rel_x,rel_y;
 	Position x,y;
 
-	if(up == -1){
+	if(options_up==-1){
 		/* first time init.. */
 		rel_x = GetXtNumber("options_popupx","Options_popupx");
 		rel_y = GetXtNumber("options_popupy","Options_popupy");
-		up = 0;
+		options_up=0;
 	}
-
-	if(up == 0)
-	{
+	if(isMapped(options_popup)==False){
 		XtTranslateCoords(toplevel,rel_x,rel_y,&x,&y);
 		XtVaSetValues(options_popup,
 		      XtNx,x,
 		      XtNy,y,
 		      NULL);
 		XtPopup(options_popup,XtGrabNone);
+		if(options_up==0){
+			setup_deletewindow(options_popup);
+			options_up=1;
+		}
 		setstatus("Bringing up options window...");
-		up = 1;
-	} else
-	{
+	} else {
 		XtPopdown(options_popup);
-		up = 0;
 	}
 }
 
-static char * freqAccel = 
- " <Key>Return:  update-frequency()";
-
-
-/* MakeFrequency:
- *  make specialised "dialog box" that takes
- *  two input values, with labels next to each input area, and
- *  one make dialog label
- */
-void MakeFrequency(Widget parent){
-	Widget mainlabel,highlabel,lowlabel;
-	XtAccelerators Accel;
-	char highvalue[10] = "";
-	char lowvalue[10] = "";
-
-	
-	
-	if(lowfrequency !=0)
-		sprintf(lowvalue,"%d",lowfrequency);
-	if(highfrequency !=0)
-		sprintf(highvalue,"%d",highfrequency);
-
-	mainlabel = XtVaCreateManagedWidget("freqLabel",labelWidgetClass,parent,
-			XtNlabel,"        Frequency Range",
-			XtNborderWidth,0,
-			NULL);
-
-	highlabel = XtVaCreateManagedWidget(
-			"freqhighLabel",labelWidgetClass,parent,
-			XtNlabel,"High:",
-			XtNfromVert,mainlabel,
-			XtNborderWidth,0,
-			NULL);
-	frequencyHigh = XtVaCreateManagedWidget(
-			"freqHigh",asciiTextWidgetClass,parent,
-			XtNfromVert,mainlabel,
-			XtNfromHoriz,highlabel,
-			XtNeditType,XawtextEdit,
-			XtNwidth,45,
-			XtNstring,highvalue,
-			NULL);
-	lowlabel = XtVaCreateManagedWidget(
-			"freqlowLabel",labelWidgetClass,parent,
-			XtNlabel,"Low:",
-			XtNfromVert,mainlabel,
-			XtNfromHoriz,frequencyHigh,
-			XtNborderWidth,0,
-			NULL);
-	frequencyLow = XtVaCreateManagedWidget(
-			"freqLow",asciiTextWidgetClass,parent,
-			XtNfromVert,mainlabel,
-			XtNfromHoriz,lowlabel,
-			XtNeditType,XawtextEdit,
-			XtNwidth,50,
-			XtNstring,lowvalue,
-			NULL);
-
-	Accel = XtParseAcceleratorTable(freqAccel);
-	XtOverrideTranslations(frequencyHigh,Accel);
-	XtOverrideTranslations(frequencyLow,Accel);
-}
-
-
-
 
 /* MakeGradeSelect()
  *	create buttons to select grades 1-6, plus "all" and "+"  buttons
@@ -249,8 +211,11 @@ void MakeGradeSelect(Widget parent){
 	XtManageChildren(gradeButtons,8);
 
 	for(widgetcount=0;widgetcount<8;widgetcount++){
-		XtAddCallback(gradeButtons[widgetcount],XtNcallback,GradeCallback,(XtPointer) widgetcount);
+		XtAddCallback(gradeButtons[widgetcount],XtNcallback,
+			      GradeCallback,(XtPointer) widgetcount);
 	}
+	/* set labels properly */
+	printgrades();
 	/* select ALL.. */
 	
 }
@@ -264,14 +229,30 @@ void changequestion(Widget w,XtPointer data,XtPointer calldata)
 	printf("changing question display\n");
 #endif
 	questionmode = buttonpressed;
+	switch(questionmode)
+	{
+		case GUESS_ENGLISH:
+			SetXtString("questionmode", "english");
+			break;
+		case GUESS_KANJI:
+			SetXtString("questionmode", "kanji");
+			break;
+		case GUESS_KANA:
+			SetXtString("questionmode", "kana");
+			break;
+	}
 	printquestion();
 }
 
 void MakeOptionsPopup()
 {
 	Widget q[3], romajibutton;
-	extern XtAccelerators AllAccel;
-
+	Widget Opshowform, Opguessform;
+	if(defaultfont!=NULL)	{
+		XCharStruct deffontsize=defaultfont->max_bounds;
+	
+		buttonswidth=12*(deffontsize.ascent+deffontsize.descent);
+	}
 
 	options_popup = XtVaCreatePopupShell("kdrill_options",
 		transientShellWidgetClass,
@@ -283,92 +264,97 @@ void MakeOptionsPopup()
 					      options_popup,
 					      NULL);
 
+	Opshowform = XtVaCreateManagedWidget("Opshowform",
+					      formWidgetClass,
+					      optionsform,
+					      NULL);
+
 
 
 	q[0] = XtVaCreateManagedWidget(
-			"q0",commandWidgetClass, optionsform,
+			"q0",commandWidgetClass, Opshowform,
 			XtNlabel,"Show kanji",
 			XtNvertDistance,8,
 			XtNshapeStyle,XawShapeOval,
-			XtNwidth,BUTTONSWIDTH,
 			NULL);
 	q[1] = XtVaCreateManagedWidget(
-			"q1",commandWidgetClass, optionsform,
+			"q1",commandWidgetClass, Opshowform,
 			XtNlabel,"Show English",
 			XtNshapeStyle,XawShapeOval,
 			XtNfromVert,q[0],
-			XtNwidth,BUTTONSWIDTH,
 			NULL);
 	q[2] = XtVaCreateManagedWidget(
-			"q2",commandWidgetClass, optionsform,
+			"q2",commandWidgetClass, Opshowform,
 			XtNlabel,"Show meaning",
 			XtNshapeStyle,XawShapeOval,
 			XtNfromVert,q[1],
-			XtNwidth,BUTTONSWIDTH,
 			NULL);
 
 
+	Opguessform = XtVaCreateManagedWidget("Opguessform",
+					      formWidgetClass,
+					      optionsform,
+					      XtNfromVert, Opshowform,
+					      NULL);
 
 
 
 	kanjimode = XtVaCreateManagedWidget(
-			"kanjimode",commandWidgetClass, optionsform,
+			"kanjimode",commandWidgetClass, Opguessform,
 			XtNlabel,"Guess kanji",
 			XtNvertDistance,8,
-			XtNfromHoriz,q[0],
 			XtNshapeStyle,XawShapeOval,
-			XtNwidth,BUTTONSWIDTH,
 			NULL);
 
 	englishmode = XtVaCreateManagedWidget(
-			"englishmode",commandWidgetClass, optionsform,
+			"englishmode",commandWidgetClass, Opguessform,
 			XtNlabel,"Guess English",
 			XtNfromVert,kanjimode,
-			XtNfromHoriz,q[0],
 			XtNshapeStyle,XawShapeOval,
-			XtNwidth,BUTTONSWIDTH,
 			NULL);
 
 	kanamode = XtVaCreateManagedWidget(
-			"kana",commandWidgetClass,optionsform,
+			"kana",commandWidgetClass, Opguessform,
 			XtNlabel,"Guess meaning",
 			XtNshapeStyle,XawShapeOval,
 			XtNfromVert,englishmode,
-			XtNfromHoriz,q[0],
-			XtNwidth,BUTTONSWIDTH,
 			NULL);
 
 
 
-	orderbutton = XtVaCreateManagedWidget("order",commandWidgetClass,optionsform,
+
+
+	orderbutton = XtVaCreateManagedWidget("order",commandWidgetClass,
+					      optionsform,
 			XtNlabel,orderlabel[(int) showinorder],
-			XtNvertDistance,8,
-			XtNfromVert,kanamode,
-			XtNwidth,BUTTONSWIDTH,
-			NULL);
-	usefilebutton = XtVaCreateManagedWidget("usefile",commandWidgetClass,optionsform,
-			XtNlabel,usefilelabel[(int) useUsefile],
-			XtNvertDistance,8,
-			XtNfromHoriz,orderbutton,
-			XtNfromVert,kanamode,
-			XtNwidth,BUTTONSWIDTH,
+			XtNfromHoriz,Opshowform,
 			NULL);
 
-	romajibutton = XtVaCreateManagedWidget("romaji",commandWidgetClass,optionsform,
+	romajibutton = XtVaCreateManagedWidget("romaji",commandWidgetClass,
+					       optionsform,
 			XtNlabel,romajilabel[(int) romajiswitch],
 			XtNfromVert,orderbutton,
-			XtNwidth,BUTTONSWIDTH,
+			XtNfromHoriz,Opshowform,
 			NULL);
 
+	usefilebutton = XtVaCreateManagedWidget("usefile",commandWidgetClass,
+						optionsform,
+			XtNlabel,usefilelabel[(int) useUsefile],
+			XtNfromVert,romajibutton,
+			XtNfromHoriz,Opshowform,
+			NULL);
+
+
+
+
+
+
 
 	gradelevelForm = XtVaCreateManagedWidget(
 			"gradeForm",formWidgetClass,optionsform,
 			XtNhorizDistance,GRADEDISTANCE,
 			XtNvertDistance,10,
-			XtNfromHoriz,usefilebutton,
-			XtNfromHoriz,kanamode,
-			XtNfromHoriz,englishmode,
-			XtNfromHoriz,usefilebutton,
+			XtNfromHoriz,orderbutton,
 			NULL);
 
 	MakeGradeSelect(gradelevelForm);
@@ -377,10 +363,9 @@ void MakeOptionsPopup()
 			"freqForm",formWidgetClass,optionsform,
 			XtNhorizDistance,GRADEDISTANCE,
 			XtNvertDistance,10,
-			XtNfromHoriz,usefilebutton,
-			XtNfromHoriz,kanamode,
-			XtNfromHoriz,englishmode,
-			XtNfromVert,gradelevelForm,
+			/*XtNfromHoriz,usefilebutton,*/
+			XtNfromHoriz,Opguessform,
+			XtNfromVert, Opshowform,
 			NULL);
 
 	MakeFrequency(frequencyForm);
@@ -393,9 +378,8 @@ void MakeOptionsPopup()
 	XtAddCallback(englishmode,XtNcallback,ChangeMode,(XtPointer) GUESS_ENGLISH);
 	XtAddCallback(kanamode,XtNcallback,ChangeMode,(XtPointer) GUESS_KANA);
 	XtAddCallback(orderbutton,XtNcallback,ordercallback,NULL);
-	XtAddCallback(usefilebutton,XtNcallback,usefilecallback,NULL);
+	XtAddCallback(usefilebutton,XtNcallback,UsefileCallback,NULL);
 	XtAddCallback(romajibutton,XtNcallback,romajicallback,NULL);
 
-	XtAugmentTranslations(options_popup,AllAccel);
 	XtSetKeyboardFocus(options_popup,toplevel);
 }
