@@ -1,6 +1,9 @@
 extern void usefilecallback(Widget,XtPointer,XtPointer);
 extern void ordercallback(Widget,XtPointer,XtPointer);
 extern void OptionsCallback(Widget,XtPointer,XtPointer);
+extern void UsefileCallback(Widget,XtPointer,XtPointer);
+extern void UpdateUsefileL();
+
 extern Widget optionsform;
 extern void MakeOptionsPopup();
 
