@@ -8,3 +8,7 @@ TRANSLATION	FindField(TRANSLATION,
 			  Boolean (*func)(TRANSLATION,void *),
 			  void *);
 int		FindIndex(char *);
+
+int		isapipe(FILE *);
+
+Boolean isMapped(Widget w);
