@@ -25,13 +25,14 @@ Widget toplevel,form;
 XFontStruct *largekfont;
 XFontStruct *smallkfont;
 XFontStruct *englishfont;
+XFontStruct *defaultfont;
 
 Widget kanjiform,englishform,buttonform;
 
 Widget statusline;
 Widget currentkanjiForm;
 Widget currentkanjiGrade,currentkanjiFreq,currentkanjiNum;
-
+Widget kanjiMissed;
 
 Widget choicesWidgets[NUMBEROFCHOICES],questionWidget;
 
@@ -95,7 +96,7 @@ int getclosestindex(int target){
 	int counter = lowestkanji-1;
 	
 	while(counter <=highestkanji){
-		while(!UseThisKanji(translations[counter])){
+		while(!UseThisKanji(counter)){
 			counter++;
 			if(counter >highestkanji){
 				setstatus("Using closest index available");
@@ -126,7 +127,7 @@ int getindexfromfreq(int freq){
 	int counter = lowestkanji-1;
 	
 	while(counter <=highestkanji){
-		while(!UseThisKanji(translations[counter])){
+		while(!UseThisKanji(counter)){
 			counter++;
 			if(counter >highestkanji)
 				return closestindex;
@@ -145,8 +146,8 @@ int getindexfromfreq(int freq){
 }
 
 /* DescribeCurrent()
- *	Sets labels to display difficulty of
- *	Current "true" kanji"
+ *	Sets labels in MAIN window to display difficulty of
+ *	reading displayed on the top
  */
 void DescribeCurrent(TRANSLATION kanji)
 {
@@ -173,7 +174,7 @@ void DescribeCurrent(TRANSLATION kanji)
 
 
 /* JumpToKanji
- *	This is the MAIN WINDOW "jump to kanji" routine
+ *	This is the MAIN WINDOW "jump to kanji" routine.
  *	The search window has its own.
  *	This handles jump for either direct index, Or Frequency.
  */
@@ -225,7 +226,8 @@ Widget MakeCurrent(Widget parent,Widget fromvert){
 
 	currentkanjiForm = XtVaCreateManagedWidget(
 			"curkanjiForm",boxWidgetClass,parent,
-			XtNright,XawChainRight,XtNleft,XawChainRight,
+			/*XtNright,XawChainRight,XtNleft,XawChainRight,*/
+			XtNleft,XawChainLeft,
 			XtNresize,False,
 			XtNfromVert,fromvert,
 			XtNorientation,XtEhorizontal,
@@ -279,9 +281,10 @@ Widget MakeCurrent(Widget parent,Widget fromvert){
 
 
 /* MakeQuestionWidget()
- *  set up kanji button widgets at initialization
+ *	set up widgets for top section of main window
  */
 void MakeQuestionWidget(){
+	Widget tmplabel;
 
 	kanjiform = XtVaCreateManagedWidget("kanjiform",formWidgetClass, form,
 			XtNright,XawChainRight,
@@ -314,6 +317,24 @@ void MakeQuestionWidget(){
 	/* now describe current kanji */
 	MakeCurrent(kanjiform,questionWidget);
 
+	tmplabel=XtVaCreateManagedWidget(
+			"missedlabel",labelWidgetClass,kanjiform,
+			XtNfromVert, questionWidget,
+			XtNfromHoriz, currentkanjiForm,
+			XtNlabel,"missed",
+			XtNborderWidth,0,
+			/* nasty hardcode.. */
+			XtNvertDistance, 10,
+			NULL);
+	kanjiMissed=XtVaCreateManagedWidget(
+			"missedkanji",asciiTextWidgetClass,kanjiform,
+			XtNfromVert, questionWidget,
+			XtNfromHoriz, tmplabel,
+			XtNwidth, INPUTWIDTH/2,
+			XtNdisplayCaret,False,
+			XtNvertDistance, 10,
+			NULL);
+
 }
 
 /* MakeEnglishButtons:
@@ -324,10 +345,11 @@ void MakeChoicesButtons(){
 	int i;
 
 
-	englishform = XtVaCreateManagedWidget("englishform",formWidgetClass,form,
-					     XtNfromVert,kanjiform,
-					     XtNvertDistance,10,
-					   NULL);
+	englishform = XtVaCreateManagedWidget("englishform",formWidgetClass,
+				form,
+				XtNfromVert,kanjiform,
+				XtNvertDistance,10,
+				NULL);
 	for(i=0;i<NUMBEROFCHOICES;i++){
 		char labl[20];
 		sprintf(labl,"english%d",i);
@@ -430,11 +452,28 @@ void MakeButtons(){
 
 void initfonts(){
 	char LargeKname[100],SmallKname[100],Efontname[100];
+	Widget tmplabel;
+	String tmpstring;
 
+	/* Sigh, okay this is ugly, tell me a better way...*/
+	/* Hell, tell me a way that WORKS, dammit... */
+	/*
+	tmplabel=XtVaCreateManagedWidget("tmplabel", labelWidgetClass,toplevel,
+				       NULL);
+	XtVaGetValues(tmplabel, "XtNfont", &defaultfont, NULL);
+	if(defaultfont==NULL){
+		puts("ERROR: cannot find default font?!!");
+		exit(0);
+	}
+	XtDestroyWidget(tmplabel);
+	*/
+	
 	GetXtString("kanjifont","Kanjifont",LargeKname);
 	GetXtString("smallkanji","Smallkanji",SmallKname);
 	GetXtString("englishfont","Englishfont",Efontname);
 
+/*	GetXtString("font", "Font", tmpstring);*/
+
 #ifdef DEBUG
 	printf("kanjifont from resources is \"%s\"\n",LargeKname);
 	printf("smallkanjifont from resources is \"%s\"\n",SmallKname);
@@ -443,6 +482,8 @@ void initfonts(){
 	largekfont = XLoadQueryFont(display,LargeKname);
 	smallkfont = XLoadQueryFont(display,SmallKname);
 	englishfont  = XLoadQueryFont(display,Efontname);
+/*	defaultfont =  XLoadQueryFont(display,tmpstring);*/
+
 	if(largekfont == NULL){
 		fprintf(stderr,"could not load large kanji font\n");
 		exit(0);
@@ -537,7 +578,7 @@ void Accelerator(Widget w,XEvent *event,String *params,Cardinal *num_params){
 			OptionsCallback(NULL,NULL,NULL);
 			break;
 		case 'u':
-			usefilecallback(usefilebutton,NULL,NULL);
+			UsefileCallback(usefilebutton,NULL,NULL);
 			break;
 		case 's':
 			SearchCallback(NULL,NULL,NULL);
@@ -555,23 +596,33 @@ void Accelerator(Widget w,XEvent *event,String *params,Cardinal *num_params){
 
 
 /* delete_calback
- *	sole purpose is to handle window deletion.
- *   still technically selected, via Actions.
- *	<Message>WM_PROTOCOLS
- *   NOT USED, (although it should work),
- *   since handle_delete() works for it. 
+ *	sole purpose is to handle window deletion. (like)<Message>WM_PROTOCOLS
+ *   still technically selected, via Actions.? bizzare, and irritating that we
+ *   have to use TWO METHODS to deal with this insanity. sigh.
+ *
+ * EXCEPT... This one seems to be to simply notify us.
+ * If we don't register this, the client dies. But now that
+ * we register this via XtAugmentTranslations... we dont actually
+ * have to do anything. Mindbogglingly stupid, really.
  */
 
 void 
 delete_callback(Widget w, XEvent *event,String *params,Cardinal *num_params)
 {
-	puts("Error! we is dead?!\n\n");
-	if(event->xclient.data.l[0] == delete_message)
-		quit(NULL, NULL, NULL);
+
+	if(event->xclient.data.l[0] == delete_message){
+		/*
+		handle_delete(w, NULL, event, NULL);
+		*/
+		return;
+	}
+	puts("Got NON-delete event, in delete_callback?");
+
 }
 
 void quitaction(Widget w, XEvent * e, String *p, Cardinal *c)
 {
+	puts("quitaction");
 	quit(NULL, NULL, NULL);
 }
 static XtActionsRec kdrillActionList[] = {
@@ -624,6 +675,7 @@ void MakeWidgets(){
 
 	MakeOptionsPopup();
 	MakeSearchPopup();
+	MakeMulti();
 	MakeButtons();
 
 #ifdef DEBUG
