@@ -1,10 +1,12 @@
 /* widgets.h */
 
 extern void DescribeCurrent(TRANSLATION);
+extern void TallyWrong();
 extern void MakeWidgets();
+extern XtAccelerators AllAccel;
 
 #ifdef USE_OKU
 extern Widget ONWidget;
 #endif
-extern Widget currentkanjiNum;
 
+extern Widget currentkanjiNum, kanjiMissed;
